==> fispro_3.4.orig.tar.gz: fispro-3.4/cpp/hfp/hfpfis.h <==
//**********************************************************************
//
//
//                              HFPFIS.H
//
//
// Author(s) : Serge GUILLAUME, Brigitte CHARNOMORDIC 
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
// File : FISHFP and OUTHFP class description, used with fispro, 
//        part of package hfp (Hierarchical fuzzy partitioning)
//        requires library fispro
//
//**********************************************************************


#ifndef __HFPFIS_H
#define __HFPFIS_H

#include "fis.h"
#include "hfpin.h"


class OUTHFP : public OUT_CRISP 
//*****************************
{
  public :
    OUTHFP(int num, double min, double max) : OUT_CRISP()
    {
      active = true;
      delete [] Name;
      Name = new char[strlen("Output ")+10];
      sprintf(Name, "Output %5d", num + 1);
      ValInf = min;
      ValSup = max;
    } 
};

//! This class does not inherit from FIS because the starting point is only 
//! a data sample file. The aim is to generate a FIS configuration file.
//! The process is made of two parts:  variable partitioning design according a 
//! type of hierachy and then rule induction. 
//! More information is available about partition design in the INHFP class. 
//! Rule conclusion initialization is achevied using the FISFPA module.
class FISHFP
//**********
{
 protected:

  INHFP ** In;
  FISOUT ** Out;

  //! System name
  char * Name;
  //! Number of inputs
  int NbIn;   
  //! Number of outputs
  int NbOut; 

  char * cConjunction;
  //! Number of examples (items) in the sample file
  int NbEx;   
  //! Number of the output to be considered. The first one is 0.
  int NSort;         
  //! Output values normalized into the unit space
  double * OutVal;  
  //! The learning data file name 
  char * fData;   
  //! The validation data file name. If NULL then fTest = fData.
  char * fTest;   
  //! The fis configuration file to be produced
  char * fFisCfg; 
  //! The vertex file name
  char * fVertex; 
  //! The rule induction method
  char * cRuleInductionMethod;

  //
  //     Partition design parameters
  //

  //! The type of hierarchy to be used to induce partitions.  
  //! Three strings are allowed:  'hfp', 'kmeans', 'regular'.
  char * strHierarchy; 
  //! Hfp parameter - Name of the merging criterion in used: 'distance' or 'variance'
  char * strMergingCrit; 
  //! Hfp parameter - Merging criterion: 0 for distance, 1 for variance  
  int MergingCriterion; 
  //! Hfp parameter - The merging criterion can be penalyzed if the merging destroy the 
  //! output heterogeneity. 'HetProp' is the tolerated proportion of impurity.
  double HetProp;  
  //! Hfp parameter - The merging criterion can be penalyzed if the merging destroy the 
  //! output heterogeneity. 'HetPenal' is the percentage of penalty to be applied when 
  //! the heterogeneity is more than 'HetProp'.
  double HetPenal;
  //! Hfp parameter - Number of Mf used to compute the sum of distances:  2, 3, 4 or -1 (all)
  int NmfDist; 
  //! Hfp parameter - The variance and density are computed for the partition with at most 
  //! 'NmfVar' Mfs.
  int NmfVar; 
  //! Hfp parameter - Name of distance in used: 'symbolic' or 'numerical'
  char * strDistanceType; 
  //! Hfp parameter - Type of distance in used: 1 for symbolic, 0 for numerical
  int Symbolic; 
  //! Hfp parameter - The numerical distance can be continuous (1) or more symbolic (0).
  //! Since July 2010, a new one is proposed, named SymbNum (-1)
  int SimpDist;    
  //! Hfp parameter - Tolerance equality threshold used to design the initial partition 
  //! from the unique values.
  double Tol; 
  //! Hfp parameter - Number of groups to be passed to the k-means algorithm to build 
  //! the initial partition.
  int NbGroup;


  //
  //     Rule conclusion initialization parameters
  //

  //! GENFIS parameter - Rules whose cumulated weight are less than this threshold 
  //! are not generated.
  double CumWeThres;   
  //! FISFPA parameter - Startegy to define Er. Allowed values:  0 for 'MIN', 1 for 'DEC'
  int CardStrategy;    
  //! FISFPA parameter - Minimum cumulated weight to define Er
  double MuMin;            
  //! FISFPA parameter - Minimum cardinality to define Er
  int MinCard;         
  
  //! For classificatiion type output
  int NbClasses;
  //! For classificatiion type output:  the misclassified array
  int * MisClass;
  //! For classificatiion type output:  the class labels
  double * Labels;


  //
  //     HFP partition selection parameters
  //

  //! Select fis option:  number of MF by variable at the beginning of the procedure. 
  //! Allowed values: 1 or 2.
  int NmfInit;
  //! Select fis option:  coverage threshold. Only systems whose coverage is greater 
  //! than this threshold are considered. 
  double BlankThres;
  //! Select fis option:  current system coverage index.
  double Coverage;
  //! Select fis option:  maximum number of iterations
  int IterMax;


  void Init(void);
  void InitSystem(char * InConfig, char * dataf, char * OutConfig);
  void InitSystem(char * dataf, char * config);
  void InitSystem(char * dataf, int ns, char * ocfg);
  void ReadHdrHfp(ifstream & f, int bufsize);
  void ReadInHfp(ifstream & f, int bufsize, int num);
  //! Identical to FIS::ReadOut but needed because FISHFP does not inherit from FIS
  void ReadOut(ifstream &f, int bufsize, int num);

  //! Call INHFP::ReadVertices.
  void ReadVertices(char * fic);

  //! The 'Strap' flag, used to generate semitrapezoidal Mf at the end of the partition, 
  //! is useful for final fis generation.
  void FisBase(int * n, char *fic, int Strap = false);
  void GenereRules(void);
  void FpaThis(void);
  void WmThis(void);
  void RuleInduction(void);
  double EvalThis(const char *fArchive, int append);

  public :

  //! This constructor is used to select a fis, generate a fis configuration file 
  //! with or without rules (see mainhfpselect.cpp, mainhfpfis.cpp and mainhfpfissr.cpp). 
  //! 'InConfig' is the input hfp configuration file. 
  //! 'dataf' is the data file name.  
  //! 'OutConfig' is the output fis configuration file. 
  FISHFP(char * InConfig, char * dataf, char * OutConfig)
    {
      InitSystem(InConfig, dataf, OutConfig);
    }

  //! This constructor is used to generate the vertex file. 
  //! 'dataf' is the data file name. 
  //! 'config' is the input hfp configuration file. 
  FISHFP(char * dataf, char * config)
    {
      InitSystem(dataf, config);
    }

  //! This constructor is used to generate the hfp configuration file. 
  //! 'dataf' is the data file name. 
  //! 'ns' is the number of variables to be considered as outputs and then 
  //! not processed at this step. 
  //! 'ocfg' is the hfp configuration file to be created. When null, 
  //! the name is 'dataf'.hfp. 
  //! 'hier' is the type of hierarchy to be written in the hfp config file, 
  //! default:  regular.
  FISHFP(char * dataf, int ns, char * ocfg = NULL, int hier = 0)
    {
      InitSystem(dataf, ns, ocfg);
      
      switch(hier)
	{
	case 1 : SetHierarchy(FISHFP::HierarchyHFP()); break;
	case 2 : SetHierarchy(FISHFP::HierarchyKMeans()); break;
	case 3 : SetHierarchy(FISHFP::HierarchyRegular()); break;
	default : SetHierarchy(FISHFP::HierarchyKMeans()); break;
	}
      PrintCfgHfp(dataf, ocfg);  
    }

  virtual ~FISHFP()
    {
      int i;
      for(i = 0; i < NbIn; i++) delete In[i];
      delete [] In;
      for(i = 0; i < NbOut; i++) delete Out[i];
      delete [] Out;

      delete [] Name;
      delete [] strDistanceType;
      delete [] strMergingCrit;
      delete [] cConjunction;
      delete [] strHierarchy;
      delete [] OutVal;
      delete [] MisClass;
      delete [] cRuleInductionMethod;
      delete [] fFisCfg;
    }

  static const char *RuleInductionFpa() { return "fpa"; }
  static const char *RuleInductionWm() { return "wm"; }

  void SetRuleInductionMethodFpa() {
    if (cRuleInductionMethod !=NULL) 
      strcpy(cRuleInductionMethod,RuleInductionFpa());
    else 
      {
	sprintf( ErrorMsg, "~ErrorInSetRuleInductionMethodFpa~");
	throw std::runtime_error( ErrorMsg );
      }
  }
  
  void SetRuleInductionMethodWm() {
    if (cRuleInductionMethod !=NULL) 
      strcpy(cRuleInductionMethod,RuleInductionWm());
    else 
      {
	sprintf( ErrorMsg, "~ErrorInSetRuleInductionMethodWm~");
	throw std::runtime_error( ErrorMsg );
      }
  }
  

  char * GetRuleInductionMethod(void) {return cRuleInductionMethod;}

  //! Call the constructor INHFP(double *t, int n, int hier, int nmf, double tol);
  //INHFP * InitOutHfp(int h, int n, int o = -1, double tol = TOLERANCE);
  FISOUT * InitOutHfp(int h, int n, int o, double tol, char *def, char *dij);
  //! Call INHFP * InitOutHfp(int h, int n, int o = -1,, double tol) 
  //! or create an OUT_CRISP(def, dij, clas) when  nmfout=0. 
  void GenereCfgFisSr(int nmfout, int hout, char *def, char *dij, double tol = TOLERANCE, int clas = false);
  //! Print a Hfp configuration file
  virtual void PrintCfgHfp(char * dataf, char * of);
  //! Print a Fis configuration file
  void PrintCfgFis(FILE * f);
 
 
  void Hfp(void);
  void HKmeans(void);
  void HRegular(void);
  void Hierarchy(char *fi = NULL);    

  void SetNSort(int s)              { NSort = s; }          // default 0
  void SetCumWeThres(double s) { CumWeThres = s; }          // default 0.0 
  void SetCardStrategy(int s)  { CardStrategy = s; }        // default DEC
  void SetMuMin(double m)           { MuMin = m; }          // default 0.3
  void SetMinCard(int e)        { MinCard = e; }            // default 3
  void SetValidationFile(char *v)   { fTest = v; }          // default data file
  void SetVertexFile(char *v)       { fVertex = v; }
  void SetBlankThres(double b)     { BlankThres = b; }      // default 1.0
  void SetNmfInit(int n)          { NmfInit = n; }          // default 1
  void SetIterMax(int i)            { IterMax = i; }        // default 10
  void SetConfigFile(char *v)       
    { 
      delete [] fFisCfg;
      fFisCfg = new char [strlen(v) +1];
      strcpy(fFisCfg, v); 
    }

  //! The Strap flag, used to generate semitrapezoidal Mf at the end of the partition,
  //! is useful for final fis generation
  void GenereCfgFis(int Strap = false);
  void SelectFis(bool wl=false);

  //! Create a result file whose format is described in the user documentation. 
  //! Fields are separated by '&' (for latex users).
  // void StoreRes(char * sc, int mr, int nr, double p, int cl, int Ap, char * fic);
  void StoreRes(char * sc, double p, double me, int cl, InfoRB & Irb, int Ap, char * fic);

  static const char *HierarchyHFP() { return "hfp"; }
  static const char *HierarchyRegular() { return "regular"; }
  static const char *HierarchyKMeans() { return "kmeans"; }

  //! Protected data handling
  void SetConjunction( const char *conjunction ); 
  char * GetConjunction() { return cConjunction; }
  char *GetHierarchy() { return strHierarchy; }
  void SetHierarchy(const char *hierarchy);
  double GetToleranceThres() { return Tol; }
  void SetToleranceThres( double tolerance ) { Tol = tolerance; }
  static const char *DistanceSymbNum() { return "symbnum"; }
  static const char *DistanceSymbolic() { return "symbolic"; }
  static const char *DistanceNumerical() { return "numerical"; }
  static const char *MergingDistance() { return "distance"; }
  static const char *MergingVariance() { return "variance"; }
  char *GetDistanceType() { return strDistanceType; }
  void SetDistanceType(const char *distance);
  char *GetMerging() { return strMergingCrit; }
  void SetMerging(const char *merging);
  int GetDistanceMFNb() { return NmfDist; }
  void SetDistanceMFNb( int distance ) { NmfDist = distance; }
  int GetVarianceMFNb() { return NmfVar; }
  void SetVarianceMFNb( int variance ) { NmfVar = variance; }
  int GetSimplifiedDistance() { return SimpDist; }
  void SetSimplifiedDistance( int simplified ) { SimpDist = simplified; }
  double GetHeteroPenalty() { return HetPenal; }
  void SetHeteroPenalty( double penalty ) { HetPenal = penalty; }
  double GetHeteroProportion() { return HetProp; }
  void SetHeteroProportion( double proportion ) { HetProp = proportion; }
  int GetNbInput() { return NbIn; }
  int GetNbOutput() { return NbOut; }
  INHFP *GetInput( int input_number ) { return In[input_number]; }
  FISOUT *GetOutput( int output_number ) { return Out[output_number]; }
  void DefaultValues();
  void ReplaceOutput( int output_number, FISOUT *output );
};

#endif

//*********************      End of HFPFIS.H      ***********************

==> fispro_3.6.orig.tar.gz: fispro-3.6/cpp/hfp/hfpfis.h <==
//**********************************************************************
//
//
//                              HFPFIS.H
//
//
// Author(s) : Serge GUILLAUME, Brigitte CHARNOMORDIC 
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
// File : FISHFP and OUTHFP class description, used with fispro, 
//        part of package hfp (Hierarchical fuzzy partitioning)
//        requires library fispro
//
//**********************************************************************


#ifndef __HFPFIS_H
#define __HFPFIS_H

#include "fis.h"
#include "hfpin.h"


class OUTHFP : public OUT_CRISP 
//*****************************
{
  public :
    OUTHFP(int num, double min, double max) : OUT_CRISP()
    {
      active = true;
      delete [] Name;
      Name = new char[strlen("Output ")+10];
      sprintf(Name, "Output %5d", num + 1);
      ValInf = min;
      ValSup = max;
    } 
};

//! This class does not inherit from FIS because the starting point is only 
//! a data sample file. The aim is to generate a FIS configuration file.
//! The process is made of two parts:  variable partitioning design according a 
//! type of hierachy and then rule induction. 
//! More information is available about partition design in the INHFP class. 
//! Rule conclusion initialization is achevied using the FISFPA module.
class FISHFP
//**********
{
 protected:

  INHFP ** In;
  FISOUT ** Out;

  //! System name
  char * Name;
  //! Number of inputs
  int NbIn;   
  //! Number of outputs
  int NbOut; 

  char * cConjunction;
  //! Number of examples (items) in the sample file
  int NbEx;   
  //! Number of the output to be considered. The first one is 0.
  int NSort;         
  //! Output values normalized into the unit space
  double * OutVal;  
  //! The learning data file name 
  char * fData;   
  //! The validation data file name. If NULL then fTest = fData.
  char * fTest;   
  //! The fis configuration file to be produced
  char * fFisCfg; 
  //! The vertex file name
  char * fVertex; 
  //! The rule induction method
  char * cRuleInductionMethod;

  //
  //     Partition design parameters
  //

  //! The type of hierarchy to be used to induce partitions.  
  //! Three strings are allowed:  'hfp', 'kmeans', 'regular'.
  char * strHierarchy; 
 //! Hfp parameter - Name of distance in used: 'symbnum', 'numerical' or  'symbolic'
  char * strDistanceType; 
  //! Hfp parameter - Type of distance in used: -1 for symbnum, 0 for numerical, 1 for symbolic
  int Symbolic; 
 //! Hfp parameter - Tolerance equality threshold used to design the initial partition 
  //! from the unique values.
  double Tol; 
  //! Hfp parameter - Number of groups to be passed to the k-means algorithm to build 
  //! the initial partition.
  int NbGroup;


  //
  //     Rule conclusion initialization parameters
  //

  //! GENFIS parameter - Rules whose cumulated weight are less than this threshold 
  //! are not generated.
  double CumWeThres;   
  //! FISFPA parameter - Startegy to define Er. Allowed values:  0 for 'MIN', 1 for 'DEC'
  int CardStrategy;    
  //! FISFPA parameter - Minimum cumulated weight to define Er
  double MuMin;            
  //! FISFPA parameter - Minimum cardinality to define Er
  int MinCard;         
  
  //! For classificatiion type output
  int NbClasses;
  //! For classificatiion type output:  the misclassified array
  int * MisClass;
  //! For classificatiion type output:  the class labels
  double * Labels;


  //
  //     HFP partition selection parameters
  //

  //! Select fis option:  number of MF by variable at the beginning of the procedure. 
  //! Allowed values: 1 or 2.
  int NmfInit;
  //! Select fis option:  coverage threshold. Only systems whose coverage is greater 
  //! than this threshold are considered. 
  double BlankThres;
  //! Select fis option:  current system coverage index.
  double Coverage;
  //! Select fis option:  maximum number of iterations
  int IterMax;


  void Init(void);
  void InitSystem(char * InConfig, char * dataf, char * OutConfig);
  void InitSystem(char * dataf, char * config);
  void InitSystem(char * dataf, int ns, char * ocfg);
  void ReadHdrHfp(ifstream & f, int bufsize);
  void ReadInHfp(ifstream & f, int bufsize, int num);
  //! Identical to FIS::ReadOut but needed because FISHFP does not inherit from FIS
  void ReadOut(ifstream &f, int bufsize, int num);

  //! Call INHFP::ReadVertices.
  void ReadVertices(char * fic);

  //! The 'Strap' flag, used to generate semitrapezoidal Mf at the end of the partition, 
  //! is useful for final fis generation.
  void FisBase(int * n, char *fic, int Strap = false);
  void GenereRules(void);
  void FpaThis(void);
  void WmThis(void);
  void RuleInduction(void);
  double EvalThis(const char *fArchive, int append);

  public :

  //! This constructor is used to select a fis, generate a fis configuration file 
  //! with or without rules (see mainhfpselect.cpp, mainhfpfis.cpp and mainhfpfissr.cpp). 
  //! 'InConfig' is the input hfp configuration file. 
  //! 'dataf' is the data file name.  
  //! 'OutConfig' is the output fis configuration file. 
  FISHFP(char * InConfig, char * dataf, char * OutConfig)
    {
      InitSystem(InConfig, dataf, OutConfig);
    }

  //! This constructor is used to generate the vertex file. 
  //! 'dataf' is the data file name. 
  //! 'config' is the input hfp configuration file. 
  FISHFP(char * dataf, char * config)
    {
      InitSystem(dataf, config);
    }

  //! This constructor is used to generate the hfp configuration file. 
  //! 'dataf' is the data file name. 
  //! 'ns' is the number of variables to be considered as outputs and then 
  //! not processed at this step. 
  //! 'ocfg' is the hfp configuration file to be created. When null, 
  //! the name is 'dataf'.hfp. 
  //! 'hier' is the type of hierarchy to be written in the hfp config file, 
  //! default:  regular.
  FISHFP(char * dataf, int ns, char * ocfg = NULL, int hier = 0)
    {
      InitSystem(dataf, ns, ocfg);
      
      switch(hier)
	{
	case 1 : SetHierarchy(FISHFP::HierarchyHFP()); break;
	case 2 : SetHierarchy(FISHFP::HierarchyKMeans()); break;
	case 3 : SetHierarchy(FISHFP::HierarchyRegular()); break;
	default : SetHierarchy(FISHFP::HierarchyKMeans()); break;
	}
      PrintCfgHfp(dataf, ocfg);  
    }

  virtual ~FISHFP()
    {
      int i;
      for(i = 0; i < NbIn; i++) delete In[i];
      delete [] In;
      for(i = 0; i < NbOut; i++) delete Out[i];
      delete [] Out;

      delete [] Name;
      delete [] strDistanceType;
      delete [] cConjunction;
      delete [] strHierarchy;
      delete [] OutVal;
      delete [] MisClass;
      delete [] cRuleInductionMethod;
      delete [] fFisCfg;
    }

  static const char *RuleInductionFpa() { return "fpa"; }
  static const char *RuleInductionWm() { return "wm"; }

  void SetRuleInductionMethodFpa() {
    if (cRuleInductionMethod !=NULL) 
      strcpy(cRuleInductionMethod,RuleInductionFpa());
    else 
      {
	sprintf( ErrorMsg, "~ErrorInSetRuleInductionMethodFpa~");
	throw std::runtime_error( ErrorMsg );
      }
  }
  
  void SetRuleInductionMethodWm() {
    if (cRuleInductionMethod !=NULL) 
      strcpy(cRuleInductionMethod,RuleInductionWm());
    else 
      {
	sprintf( ErrorMsg, "~ErrorInSetRuleInductionMethodWm~");
	throw std::runtime_error( ErrorMsg );
      }
  }
  

  char * GetRuleInductionMethod(void) {return cRuleInductionMethod;}

  //! Call the constructor INHFP(double *t, int n, int hier, int nmf, double tol);
  //INHFP * InitOutHfp(int h, int n, int o = -1, double tol = TOLERANCE);
  FISOUT * InitOutHfp(int h, int n, int o, double tol, char *def, char *dij);
  //! Call INHFP * InitOutHfp(int h, int n, int o = -1,, double tol) 
  //! or create an OUT_CRISP(def, dij, clas) when  nmfout=0. 
  void GenereCfgFisSr(int nmfout, int hout, char *def, char *dij, double tol = TOLERANCE, int clas = false);
  //! Print a Hfp configuration file
  virtual void PrintCfgHfp(char * dataf, char * of);
  //! Print a Fis configuration file
  void PrintCfgFis(FILE * f);
 
 
  void Hfp(void);
  void HKmeans(void);
  void HRegular(void);
  void Hierarchy(char *fi = NULL);    

  void SetNSort(int s)              { NSort = s; }          // default 0
  void SetCumWeThres(double s) { CumWeThres = s; }          // default 0.0 
  void SetCardStrategy(int s)  { CardStrategy = s; }        // default DEC
  void SetMuMin(double m)           { MuMin = m; }          // default 0.3
  void SetMinCard(int e)        { MinCard = e; }            // default 3
  void SetValidationFile(char *v)   { fTest = v; }          // default data file
  void SetVertexFile(char *v)       { fVertex = v; }
  void SetBlankThres(double b)     { BlankThres = b; }      // default 1.0
  void SetNmfInit(int n)          { NmfInit = n; }          // default 1
  void SetIterMax(int i)            { IterMax = i; }        // default 10
  void SetConfigFile(char *v)       
    { 
      delete [] fFisCfg;
      fFisCfg = new char [strlen(v) +1];
      strcpy(fFisCfg, v); 
    }

  //! The Strap flag, used to generate semitrapezoidal Mf at the end of the partition,
  //! is useful for final fis generation
  void GenereCfgFis(int Strap = false);
  void SelectFis(bool wl=false);

  //! Create a result file whose format is described in the user documentation. 
  //! Fields are separated by '&' (for latex users).
  // void StoreRes(char * sc, int mr, int nr, double p, int cl, int Ap, char * fic);
  void StoreRes(char * sc, double p, double me, int cl, InfoRB & Irb, int Ap, char * fic);

  static const char *HierarchyHFP() { return "hfp"; }
  static const char *HierarchyRegular() { return "regular"; }
  static const char *HierarchyKMeans() { return "kmeans"; }

  //! Protected data handling
  void SetConjunction( const char *conjunction ); 
  char * GetConjunction() { return cConjunction; }
  char *GetHierarchy() { return strHierarchy; }
  void SetHierarchy(const char *hierarchy);
  double GetToleranceThres() { return Tol; }
  void SetToleranceThres( double tolerance ) { Tol = tolerance; }
  static const char *DistanceSymbNum() { return "symbnum"; }
  static const char *DistanceSymbolic() { return "symbolic"; }
  static const char *DistanceNumerical() { return "numerical"; }
  char *GetDistanceType() { return strDistanceType; }
  void SetDistanceType(const char *distance);
  int GetNbInput() { return NbIn; }
  int GetNbOutput() { return NbOut; }
  INHFP *GetInput( int input_number ) { return In[input_number]; }
  FISOUT *GetOutput( int output_number ) { return Out[output_number]; }
  void DefaultValues();
  void ReplaceOutput( int output_number, FISOUT *output );
};

#endif

//*********************      End of HFPFIS.H      ***********************
